import asyncio
import threading
import time
from .request import CommandMode, Request
from .const import StoveCommands, StoveRegisters, StoveState, StoveManufacturer
from .client import HottohRemoteClient
import logging
from bitstring import BitArray

# _LOGGER = logging.getLogger(__name__)

MAX_CONNECTION_RETRIES = 3

class HottohConnectionError(Exception):
    """Hottoh connection exception."""

    pass

class Hottoh:
    def __init__(self, address, port, id=0):
        """Communicate with HottoH stove wifi module"""
        self.log = logging.getLogger(__name__)
        self.port = port
        self.address = address
        self.client = HottohRemoteClient(self.address, self.port, id)

    def connect(self):
        self.client.start()

    def disconnect(self):
        self.client.stop()

    def is_connected(self):
        return self.client.is_connected()

    def _extractData(self, data):
        # Split data to an array
        return data.split(";")
            
    def _getMac(self):
        return "aabbccddeeff"

    def set_temperature(self, value):
        """Set Target Temperature of the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[
                    str(StoveCommands.PARAM_AMBIANCE_TEMPERATURE_1.value),
                    str(value * 10),
                ]
            )
            return result
        except:
            raise

    def set_water_temperature(self, value):
        """Set Target Water Temperature of the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[
                    str(StoveCommands.INCONNU_4.value),
                    str(value * 10),
                ]
            )
            return result
        except:
            raise

    def set_power_level(self, value):
        """Set power level of the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_NIVEAU_PUISSANCE.value), str(value)]
            )
            return result
        except:
            raise

    def set_speed_fan_1(self, value):
        """Set speed level of Fan 1"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_NIVEAU_FAN_1.value), str(value)]
            )
            return result
        except:
            raise

    def set_speed_fan_2(self, value):
        """Set speed level of Fan 2"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_NIVEAU_FAN_2.value), str(value)]
            )
            return result
        except:
            raise

    def set_speed_fan_3(self, value):
        """Set speed level of Fan 3"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_NIVEAU_FAN_3.value), str(value)]
            )
            return result
        except:
            raise

    def set_eco_mode_on(self):
        """Set Eco Mode of the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_ECO_MODE.value), "1"]
            )
            return result
        except:
            raise

    def set_eco_mode_off(self):
        """Set Eco Mode of the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_ECO_MODE.value), "0"]
            )
            return result
        except:
            raise

    def set_chrono_mode_on(self):
        """Set Eco Mode of the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_CHRONO_ON_OFF.value), "1"]
            )
            return result
        except:
            raise

    def set_chrono_mode_off(self):
        """Set Eco Mode of the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_CHRONO_ON_OFF.value), "0"]
            )
            return result
        except:
            raise

    def set_on(self):
        """Set On/ Off  the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_ON_OFF.value), "1"]
            )
            return result
        except:
            raise

    def set_off(self):
        """Set On/ Off  the stove"""
        try:
            result = self.client.sendCommand(
                parameters=[str(StoveCommands.PARAM_ON_OFF.value), "0"]
            )
            return result
        except:
            raise

    def get_smoke_temperature(self):
        return float(self._getSmokeTemperature())

    def get_water_temperature(self):
        return float(self._getTemperatureWater())

    def get_set_water_temperature(self):
        return float(self._getSetTemperatureWater())

    def get_set_min_water_temperature(self):
        return float(self._getSetMinTemperatureWater())

    def get_set_max_water_temperature(self):
        return float(self._getSetMaxTemperatureWater())

    def get_speed_fan_smoke(self):
        return float(self._getSpeedFanSmoke())

    def get_temperature_room_1(self):
        return float(self._getTemperatureRoom1())
    
    def get_set_temperature_room_1(self):
        return float(self._getSetTemperatureRoom1())

    def get_set_min_temperature_room_1(self):
        return float(self._getSetMinTemperatureRoom1())

    def get_set_max_temperature_room_1(self):
        return float(self._getSetMaxTemperatureRoom1())

    def get_temperature_room_2(self):
        return float(self._getTemperatureRoom2())
    
    def get_set_temperature_room_2(self):
        return float(self._getSetTemperatureRoom2())

    def get_set_min_temperature_room_2(self):
        return float(self._getSetMinTemperatureRoom2())

    def get_set_max_temperature_room_2(self):
        return float(self._getSetMaxTemperatureRoom2())

    def get_temperature_room_3(self):
        return float(self._getTemperatureRoom3())
    
    def get_set_temperature_room_3(self):
        return float(self._getSetTemperatureRoom3())

    def get_set_min_temperature_room_3(self):
        return float(self._getSetMinTemperatureRoom3())

    def get_set_max_temperature_room_3(self):
        return float(self._getSetMaxTemperatureRoom3())

    def get_speed_fan_1(self):
        return int(self._getSpeedFan1())

    def get_set_speed_fan_1(self):
        return int(self._getSetSpeedFan1())

    def get_set_max_speed_fan_1(self):
        return int(self._getSetMaxSpeedFan1())

    def get_speed_fan_2(self):
        return float(self._getSpeedFan2())

    def get_set_speed_fan_2(self):
        return float(self._getSetSpeedFan2())

    def get_set_max_speed_fan_2(self):
        return float(self._getSetMaxSpeedFan2())

    def get_speed_fan_3(self):
        return float(self._getSpeedFan3())

    def get_set_speed_fan_3(self):
        return float(self._getSetSpeedFan3())

    def get_set_max_speed_fan_3(self):
        return float(self._getSetMaxSpeedFan3()) 

    def get_power_level(self):
        return float(self._getPowerLevel())
    
    def get_set_power_level(self):
        return float(self._getSetPowerLevel())

    def get_set_min_power_level(self):
        return float(self._getSetMinPowerLevel())

    def get_set_max_power_level(self):
        return float(self._getSetMaxPowerLevel())

    def get_air_ex_1(self):
        return float(self._getAirEx1())

    def get_air_ex_2(self):
        return float(self._getAirEx2())

    def get_air_ex_3(self):
        return float(self._getAirEx3())

    def get_action(self):
        if self._getStoveState() in ['switched_off', 'black_out', 'eco_stop_2', 'eco_stop_3']:
            return 'off'
        if self._getStoveState() in ['starting_1_check']:
            return 'check'
        if self._getStoveState() in ['starting_2_clean_all']:
            return 'clean_all'
        if self._getStoveState() in ['starting_3_loading']:
            return 'loading'
        if self._getStoveState() in ['starting_4_waiting', 'starting_5_waiting']:
            return 'waiting'
        if self._getStoveState() in ['starting_6_ignition']:
            return 'ignition'
        if self._getStoveState() in ['starting_7_stabilization']:
            return 'stabilization'
        if self._getStoveState() in ['power']:
            return 'heating'
        if self._getStoveState() in ['stopping_1_wait_standby', 'stopping_2_wait_standby']:
            return 'stopping'
        if self._getStoveState() in ['eco_stop_1_standby', 'standby']:
            return 'idle'
        return self._getStoveState()

    def get_is_on(self):
        return self._getStoveIsOn() == 'on'

    def get_eco_mode(self):
        return self._getEcoMode() == 'on'

    def get_chrono_mode(self):
        return self._getChronoMode() == 'on'
        
    def get_mode(self):
        return self._getStoveIsOn()

    def get_name(self):
        if self.get_manufacturer() is None:
            return None
        return "Stove " + self.get_manufacturer()
        
    def get_firmware(self):
        return self._getFirmwareVersion()

    def get_wifi(self):
        return self._getWifiSignal()

    def get_manufacturer(self):
        return self._getManufacturer()

    def get_water_pump(self):
        if int(self._getGenericPump()) == 0:
            return False
        return True

    def _getFirmwareVersion(self):
        if self.client._info is None:
            return None
        return self.client._info[StoveRegisters.INDEX_FW]

    def _getWifiSignal(self):
        if self.client._info is None:
            return None
        return self.client._info[StoveRegisters.INDEX_WIFI]

    def _getPageIndex(self):
        if self.client._data is None:
            return None
        return self.client._data[StoveRegisters.INDEX_PAGE]

    def _getManufacturer(self):
        if self.client._data is None:
            return None
        if int(self.client._data[StoveRegisters.INDEX_MANUFACTURER]) == StoveManufacturer.STOVE_MANUFACTURER_CMG:
            return "CMG"
        if int(self.client._data[StoveRegisters.INDEX_MANUFACTURER]) == StoveManufacturer.STOVE_MANUFACTURER_EDILKAMIN:
            return "EdilKamin"
        return str(self.client._data[StoveRegisters.INDEX_MANUFACTURER])

    def _getIsBitmapVisible(self):
        if self.client._data is None:
            return None
        return self.client._data[StoveRegisters.INDEX_BITMAP_VISIBLE]

    def _getIsValid(self):
        if self.client._data is None:
            return None
        return self.client._data[StoveRegisters.INDEX_VALID]

    def _getStoveType(self):
        if self.client._data is None:
            return 0
        return int(self.client._data[StoveRegisters.INDEX_STOVE_TYPE])

    def _getStoveState(self):
        if self.client._data is None:
            return "not_connected"
        if StoveState.STATUS_OFF == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "switched_off"
        if StoveState.STATUS_STARTING_1 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "starting_1_check"
        if StoveState.STATUS_STARTING_2 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "starting_2_clean_all"
        if StoveState.STATUS_STARTING_3 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "starting_3_loading"
        if StoveState.STATUS_STARTING_4 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "starting_4_waiting"
        if StoveState.STATUS_STARTING_5 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "starting_5_waiting"
        if StoveState.STATUS_STARTING_6 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "starting_6_ignition"
        if StoveState.STATUS_STARTING_7 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "starting_7_stabilization"
        if StoveState.STATUS_POWER == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "power"
        if StoveState.STATUS_STOPPING_1 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "stopping_1_wait_standby"
        if StoveState.STATUS_STOPPING_2 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "stopping_2_wait_standby"
        if StoveState.STATUS_ECO_STOP_1 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "eco_stop_1_standby"
        if StoveState.STATUS_ECO_STOP_2 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "eco_stop_2"
        if StoveState.STATUS_ECO_STOP_3 == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "eco_stop_3"
        if StoveState.STATUS_LOW_PELLET == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "low_pellet"
        if StoveState.STATUS_END_PELLET == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "end_pellet"
        if StoveState.STATUS_BLACK_OUT == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "black_out"
        if StoveState.STATUS_INGNITION_FAILED == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "error_ignition_failed"
        if StoveState.STATUS_ANTI_FREEZE == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "anti_freeze"
        if StoveState.STATUS_COVER_OPEN == int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "error_cover_open"
        if StoveState.STATUS_NO_PELLET== int(self.client._data[StoveRegisters.INDEX_STOVE_STATE]):
            return "error_no_pellet"
        return str(self.client._data[StoveRegisters.INDEX_STOVE_STATE])

    def _getStoveIsOn(self):
        if self.client._data is None:
            return None
        if self.client._data[StoveRegisters.INDEX_STOVE_ON] == "1":
            return "on"
        else:
            return "off"

    def _getEcoMode(self):
        if self.client._data is None:
            return None
        if self.client._data[StoveRegisters.INDEX_ECO_MODE] == "1":
            return "on"
        else:
            return "off"

    def _getChronoMode(self):
        if self.client._data is None:
            return None
        if self.client._data[StoveRegisters.INDEX_TIMER_ON] == "1":
            return "on"
        else:
            return "off"

    def _getTemperatureRoom1(self):
        if self.client._data is not None:
            return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T1]) / 10)
        return 0

    def _getSetTemperatureRoom1(self):
        if self.client._data is not None:
            return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T1_SET]) / 10)
        return 0

    def _getSetMaxTemperatureRoom1(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T1_SET_MAX]) / 10)

    def _getSetMinTemperatureRoom1(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T1_SET_MIN]) / 10)

    def _getTemperatureRoom2(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T2]) / 10)

    def _getSetTemperatureRoom2(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T2_SET]) / 10)

    def _getSetMaxTemperatureRoom2(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T2_SET_MAX]) / 10)

    def _getSetMinTemperatureRoom2(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_AMBIENT_T2_SET_MIN]) / 10)

    def _getTemperatureRoom3(self):
        if self.client._data2 is None:
            return None
        return str(int(self.client._data2[StoveRegisters.INDEX_ROOM_TEMP_3]) / 10)

    def _getSetTemperatureRoom3(self):
        if self.client._data2 is None:
            return None
        return str(int(self.client._data2[StoveRegisters.INDEX_ROOM_TEMP_3_SET]) / 10)

    def _getSetMinTemperatureRoom3(self):
        if self.client._data2 is None:
            return None
        return str(int(self.client._data2[StoveRegisters.INDEX_ROOM_TEMP_3_SET_MIN]) / 10)

    def _getSetMaxTemperatureRoom3(self):
        if self.client._data2 is None:
            return None
        return str(int(self.client._data2[StoveRegisters.INDEX_ROOM_TEMP_3_SET_MAX]) / 10)

    def _getTemperatureWater(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_WATER]) / 10)

    def _getSetTemperatureWater(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_WATER_SET]) / 10)

    def _getSetMaxTemperatureWater(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_WATER_SET_MAX]) / 10)

    def _getSetMinTemperatureWater(self):
        if self.client._data is None:
            return 0
        return str(int(self.client._data[StoveRegisters.INDEX_WATER_SET_MIN]) / 10)

    def _getSmokeTemperature(self):
        if self.client._data is not None:
            return str(int(self.client._data[StoveRegisters.INDEX_SMOKE_T]) / 10)
        return 0

    def _getPowerLevel(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_POWER_LEVEL]

    def _getSetPowerLevel(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_POWER_SET]

    def _getSetMaxPowerLevel(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_POWER_MAX]

    def _getSetMinPowerLevel(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_POWER_MIN]

    def _getSpeedFanSmoke(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_SMOKE]

    def _getSpeedFan1(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_1]

    def _getSetSpeedFan1(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_1_SET]

    def _getSetMaxSpeedFan1(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_1_SET_MAX]

    def _getSpeedFan2(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_2]

    def _getSetSpeedFan2(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_2_SET]

    def _getSetMaxSpeedFan2(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_2_SET_MAX]

    def _getSpeedFan3(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_3]

    def _getSetSpeedFan3(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_3_SET]

    def _getSetMaxSpeedFan3(self):
        if self.client._data is None:
            return 0
        return self.client._data[StoveRegisters.INDEX_FAN_3_SET_MAX]

## Datapoints from data2
    def _getFlowSwitch(self):
        if self.client._data2 is None:
            return 0
        return self.client._data2[StoveRegisters.INDEX_FLOW_SWITCH]

    def _getGenericPump(self):
        if self.client._data2 is None:
            return 0
        return self.client._data2[StoveRegisters.INDEX_GENERIC_PUMP]

    def _getAirEx1(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_AIREX_1]

    def _getAirEx2(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_AIREX_2]

    def _getAirEx3(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_AIREX_3]

    def _getPuffer(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_PUFFER]

    def _getSetPuffer(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_PUFFER_SET]

    def _getSetMinPuffer(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_PUFFER_SET_MIN]

    def _getSetMaxPuffer(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_PUFFER_SET_MAX]

    def _getBoiler(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_BOILER]

    def _getSetBoiler(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_BOILER_SET]

    def _getSetMinBoiler(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_BOILER_SET_MIN]

    def _getSetMaxBoiler(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_BOILER_SET_MAX]

    def _getDhw(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_DHW]

    def _getSetDhw(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_DHW_SET]

    def _getSetMinDhw(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_DHW_SET_MIN]

    def _getSetMaxDhw(self):
        if self.client._data2 is None:
            return None
        return self.client._data2[StoveRegisters.INDEX_DHW_SET_MAX]

    def _getStoveTypeBitArray(self):
        return BitArray(uint=self._getStoveType(), length=16)

    def isBoilerEnabled(self):
        type = self._getStoveTypeBitArray()
        return type[9]

    def isDomesticHotWaterEnabled(self):
        type = self._getStoveTypeBitArray()
        return type[10]

    def getFanNumber(self):
        type = self._getStoveTypeBitArray()
        nb = type[12:14]
        return nb.int
    
    def isTempRoom1Enabled(self):
        type = self._getStoveTypeBitArray()
        return type[15]

    def isTempRoom2Enabled(self):
        type = self._getStoveTypeBitArray()
        return type[8]

    def isTempRoom3Enabled(self):
        type = self._getStoveTypeBitArray()
        return type[7]

    def isTempWaterEnabled(self):
        type = self._getStoveTypeBitArray()
        return type[14]

    def isPumpEnabled(self):
        type = self._getStoveTypeBitArray()
        return type[4]




from enum import Enum

import crcmod.predefined


class Command(Enum):
    DAT = "DAT"


class CommandMode(Enum):
    READ = "R"
    WRITE = "W"
    EXECUTE = "E"

IdSocket = 0

class Request:
    def __init__(self, command="DAT", mode="R", parameters=["0"], id=0):
        """Generate communication frame"""
        self.command = command
        self.mode = mode
        self.parameters = parameters
        self.id = id
        self.request = ""
        self._buildRawPacket()

    def _buildRawPacket(self):
        """Buld raw packet from data"""
        strSocketId = str(0).zfill(5)
        strRawData = self._getRawData()
        strData = strSocketId + "C---" + strRawData
        strCrc = self._getCrc(strData)
        self.request = "#" + strData + strCrc + "\n"

    def _getRawData(self):
        lenParameters = f"{len(self._getParameters()):0>4X}"
        strCmd = lenParameters + self.command + self.mode
        return strCmd + self._getParameters()

    def _getParameters(self):
        str1 = ""
        for value in self.parameters:
            str1 += value + ";"

        return str1

    def _getCrc(self, message):
        crc16 = crcmod.predefined.Crc("crc-ccitt-false")
        crc16.update(message.encode("utf-8"))
        return crc16.hexdigest()

    def getRequest(self):
        return self.request.encode("utf-8")



from enum import IntEnum


class StoveState(IntEnum):
    STATUS_OFF = 0
    STATUS_STARTING_1 = 1
    STATUS_STARTING_2 = 2
    STATUS_STARTING_3 = 3
    STATUS_STARTING_4 = 4
    STATUS_STARTING_5 = 5
    STATUS_STARTING_6 = 6
    STATUS_STARTING_7 = 7
    STATUS_POWER = 8
    STATUS_STOPPING_1 = 9
    STATUS_STOPPING_2 = 10
    STATUS_ECO_STOP_1 = 11
    STATUS_ECO_STOP_2 = 12
    STATUS_ECO_STOP_3 = 13
    STATUS_LOW_PELLET = 14
    STATUS_END_PELLET = 15
    STATUS_BLACK_OUT = 16
    STATUS_ANTI_FREEZE = 17
    STATUS_INGNITION_FAILED = 60
    STATUS_NO_PELLET = 61
    STATUS_COVER_OPEN = 69


class StoveChronoMode(IntEnum):
    CHRONO_OFF = 0
    CHRONO_SLEEP = 1
    CHRONO_ON_1 = 2
    CHRONO_ON_2 = 3
    CHRONO_ON_3 = 4
    CHRONO_ON_4 = 5


class StoveManufacturer(IntEnum):
    STOVE_MANUFACTURER_CMG = 9
    STOVE_MANUFACTURER_65 = 65
    STOVE_MANUFACTURER_76 = 76
    STOVE_MANUFACTURER_EDILKAMIN = 85
    STOVE_MANUFACTURER_100 = 100


class StoveRegisters(IntEnum):
    ## Infos ##
    INDEX_FW = 1
    INDEX_WIFI = 2

    ## Data 0 ##
    INDEX_PAGE = 0
    INDEX_MANUFACTURER = 1
    INDEX_BITMAP_VISIBLE = 2
    INDEX_VALID = 3
    INDEX_STOVE_TYPE = 4
    INDEX_STOVE_STATE = 5
    INDEX_STOVE_ON = 6
    INDEX_ECO_MODE = 7
    INDEX_TIMER_ON = 8
    INDEX_AMBIENT_T1 = 9
    INDEX_AMBIENT_T1_SET = 10
    INDEX_AMBIENT_T1_SET_MIN = 11
    INDEX_AMBIENT_T1_SET_MAX = 12
    INDEX_AMBIENT_T2 = 13
    INDEX_AMBIENT_T2_SET = 14
    INDEX_AMBIENT_T2_SET_MIN = 15
    INDEX_AMBIENT_T2_SET_MAX = 16
    INDEX_WATER = 17
    INDEX_WATER_SET = 18
    INDEX_WATER_SET_MIN = 19
    INDEX_WATER_SET_MAX = 20
    INDEX_SMOKE_T = 21
    INDEX_POWER_LEVEL = 22
    INDEX_POWER_SET = 23
    INDEX_POWER_MIN = 24
    INDEX_POWER_MAX = 25
    INDEX_FAN_SMOKE = 26
    INDEX_FAN_1 = 27
    INDEX_FAN_1_SET = 28
    INDEX_FAN_1_SET_MAX = 29
    INDEX_FAN_2 = 30
    INDEX_FAN_2_SET = 31
    INDEX_FAN_2_SET_MAX = 32
    INDEX_FAN_3 = 33
    INDEX_FAN_3_SET = 34
    INDEX_FAN_3_SET_MAX = 35

    ## Data 1 Chrono ##
    INDEX_STATE = 1
    INDEX_TEMPERATURE_1 = 2
    INDEX_TEMPERATURE_1_MIN = 3
    INDEX_TEMPERATURE_1_MAX = 4
    INDEX_TEMPERATURE_2 = 5
    INDEX_TEMPERATURE_2_MIN = 6
    INDEX_TEMPERATURE_2_MAX = 7
    INDEX_TEMPERATURE_3 = 8
    INDEX_TEMPERATURE_3_MIN = 9
    INDEX_TEMPERATURE_3_MAX = 10

    ## Data 2 ##
    INDEX_FLOW_SWITCH = 1
    INDEX_GENERIC_PUMP = 2
    INDEX_AIREX_1 = 3
    INDEX_AIREX_2 = 4
    INDEX_AIREX_3 = 5
    INDEX_PUFFER = 6
    INDEX_PUFFER_SET = 7
    INDEX_PUFFER_SET_MIN = 8
    INDEX_PUFFER_SET_MAX = 9
    INDEX_BOILER = 10
    INDEX_BOILER_SET = 11
    INDEX_BOILER_SET_MIN = 12
    INDEX_BOILER_SET_MAX = 13
    INDEX_DHW = 14
    INDEX_DHW_SET = 15
    INDEX_DHW_SET_MIN = 16
    INDEX_DHW_SET_MAX = 17
    INDEX_ROOM_TEMP_3 = 18
    INDEX_ROOM_TEMP_3_SET = 19
    INDEX_ROOM_TEMP_3_SET_MIN = 20
    INDEX_ROOM_TEMP_3_SET_MAX = 21


class StoveCommands(IntEnum):
    PARAM_ON_OFF = 0
    PARAM_ECO_MODE = 1
    PARAM_NIVEAU_PUISSANCE = 2
    PARAM_AMBIANCE_TEMPERATURE_1 = 3
    INCONNU_4 = 4
    PARAM_NIVEAU_FAN_1 = 5
    PARAM_NIVEAU_FAN_2 = 6
    PARAM_NIVEAU_FAN_3 = 7
    PARAM_CHRONO_ON_OFF = 8
    PARAM_CHRONO_TEMPERATURE_1 = 9
    PARAM_CHRONO_TEMPERATURE_2 = 10
    PARAM_CHRONO_TEMPERATURE_3 = 11



import logging
import threading
import socket
import errno
import time
from .request import Request

MAX_CONNECTION_RETRIES = 3


class HottohRemoteClient:
    address = None
    port = None
    _log = None
    _info = None
    _data = None
    _data2 = None
    _raw = None
    _write_request = False
    _write_parameters = []
    _disconnect_request = False
    _socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    __is_connected = False
    __end_request = False

    def __init__(self, address="192.168.4.10", port=5001, id=0):
        """Create tcp client."""
        self.id = id
        self.address = address
        self.port = port
        self._log = logging.getLogger(__name__)
        self._thread = threading.Thread(
            target= self.loop, name="hottohpy-client"
        )

    def check(self):
        if self.connect():
            data = self._get_data(command="DAT", parameters=["0"])
            self.socket.close()
            self.was_connected = False
            if len(data) > 0:
                return True
            return False
        return False

    def is_connected(self):
        return self.__is_connected

    def _get_data(self, command, parameters):
        request = Request(command=command, parameters=parameters, id=self.id)
        self._socket.send(request.getRequest())
        self._socket.settimeout(60)
        data = self._socket.recv(1024)
        return self._extractData(f"{data}")

    def _set_data(self, parameters):
        request = Request(command="DAT", mode="W", parameters=parameters, id=self.id)
        self._socket.send(request.getRequest())
        self._socket.settimeout(60)
        data = self._socket.recv(1024)
        return self._extractData(f"{data}")
    
    def _set_raw(self, command, mode, parameters):
        request = Request(command=command, mode=mode, parameters=parameters, id=self.id)
        print(request.getRequest())
        self._socket.send(request.getRequest())
        self._socket.settimeout(60)
        data = self._socket.recv(1024)
        print(data)
        return self._extractData(f"{data}")
            
    def _extractData(self, data):
        # Split data to an array
        return data.split(";")

    def sendCommand(self, parameters):
        self._write_request = True
        self._write_parameters.append(parameters)
        return True

    def start(self):
        if not self._thread.is_alive():
            self._log.debug("Start thread to communicate with the stove")
            if self._thread.daemon is not True:
                self._thread.daemon = True
            self._thread.start()

    def stop(self):
        self.__end_request = True

    def loop(self):

        while self.__end_request is False:
            ## Start connection to the stove
            if self.__is_connected is False:
                self.__connect()
            ## if connection ok -> start fetching data's
            ## if not retry forever or until end request

            ## Fetch data
            ## Write data if needed
            if self.__is_connected is True:
                self.__dial()

        ## if end request -> disconnect properly
        ## Disconnect
        self.__disconnect()
           
    def __connect(self):
        try:
            self._log.debug("Try to connect the Stove")
            self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self._socket.connect((self.address, self.port))
            self.__is_connected = True
        except socket.error as error:
            self.__is_connected = False
            if error.errno == errno.EISCONN:
                self._log.error("Socket already connected to %s", self.address)
                time.sleep(5)
                pass

            self._log.error("Socket error: %s", error)
            time.sleep(5)
            pass
            
    def __dial(self):
        try:
            # self._log.debug("Fetching stove data's")
            # Get info
            self._info = self._get_data("INF", [""])
            # self._log.debug("Information Data %s", self._info)
            # Get Data
            self._data = self._get_data("DAT", ["0"])
            # self._log.debug("Information Data %s", self._data)
            self._data2 = self._get_data("DAT", ["2"])
            # Write if needed
            while len(self._write_parameters):
                param = self._write_parameters[0]
                self._log.debug("Send Command %s", param)
                res = self._set_data(param)
                self._log.debug("Get Response command %s", res)
                self._write_parameters.remove(param)

            self._write_parameters = []
            self._write_request = False
            
            time.sleep(1)

        except socket.error as error:
            self.__is_connected = False
            self._log.error("Dial Error : %s", error)
            pass

    def __disconnect(self):
        self.__is_connected = False
        self._log.debug("Close Stove connection")
        self._socket.close()



import setuptools
import os
import codecs
import re

# Method for retrieving the version is taken from the setup.py of pip itself:
# https://github.com/pypa/pip/blob/master/setup.py
here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hottohpy",
    version=find_version("src/hottohpy", "__init__.py"),
    author="benlbrm",
    author_email="web@lebruman.fr",
    description="Python lib to control Hottoh based stove Devices",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/benlbrm/hottohpy",
    project_urls={
        "Bug Tracker": "https://github.com/benlbrm/hottohpy/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Home Automation',
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)
